@@ -14,12 +14,16 @@
 
 # This class contains the basic functionality needed to run any interpreter
 # or an interpreter-based tool.
+from __future__ import annotations
 
-from ..mesonlib import MesonException
+from ..mesonlib import MesonException, OptionKey
 from .. import mlog
-from .._pathlib import Path
+from pathlib import Path
 import typing as T
 
+if T.TYPE_CHECKING:
+    from ..environment import Environment
+
 language_map = {
     'c': 'C',
     'cpp': 'CXX',
@@ -32,6 +36,18 @@ language_map = {
     'swift': 'Swift',
 }
 
+backend_generator_map = {
+    'ninja': 'Ninja',
+    'xcode': 'Xcode',
+    'vs2010': 'Visual Studio 10 2010',
+    'vs2012': 'Visual Studio 11 2012',
+    'vs2013': 'Visual Studio 12 2013',
+    'vs2015': 'Visual Studio 14 2015',
+    'vs2017': 'Visual Studio 15 2017',
+    'vs2019': 'Visual Studio 16 2019',
+    'vs2022': 'Visual Studio 17 2022',
+}
+
 blacklist_cmake_defs = [
     'CMAKE_TOOLCHAIN_FILE',
     'CMAKE_PROJECT_INCLUDE',
@@ -46,6 +62,18 @@ blacklist_cmake_defs = [
     'MESON_CMAKE_ROOT',
 ]
 
+def cmake_is_debug(env: 'Environment') -> bool:
+    if OptionKey('b_vscrt') in env.coredata.options:
+        is_debug = env.coredata.get_option(OptionKey('buildtype')) == 'debug'
+        if env.coredata.options[OptionKey('b_vscrt')].value in {'mdd', 'mtd'}:
+            is_debug = True
+        return is_debug
+    else:
+        # Don't directly assign to is_debug to make mypy happy
+        debug_opt = env.coredata.get_option(OptionKey('debug'))
+        assert isinstance(debug_opt, bool)
+        return debug_opt
+
 class CMakeException(MesonException):
     pass
 
@@ -56,7 +84,7 @@ class CMakeBuildFile:
         self.is_temp = is_temp
 
     def __repr__(self) -> str:
-        return '<{}: {}; cmake={}; temp={}>'.format(self.__class__.__name__, self.file, self.is_cmake, self.is_temp)
+        return f'<{self.__class__.__name__}: {self.file}; cmake={self.is_cmake}; temp={self.is_temp}>'
 
 def _flags_to_list(raw: str) -> T.List[str]:
     # Convert a raw commandline string into a list of strings
@@ -73,9 +101,9 @@ def _flags_to_list(raw: str) -> T.List[str]:
             escape = False
         elif i == '\\':
             escape = True
-        elif i in ['"', "'"]:
+        elif i in {'"', "'"}:
             in_string = not in_string
-        elif i in [' ', '\n']:
+        elif i in {' ', '\n'}:
             if in_string:
                 curr += i
             else:
@@ -84,9 +112,15 @@ def _flags_to_list(raw: str) -> T.List[str]:
         else:
             curr += i
     res += [curr]
-    res = list(filter(lambda x: len(x) > 0, res))
+    res = [r for r in res if len(r) > 0]
     return res
 
+def cmake_get_generator_args(env: 'Environment') -> T.List[str]:
+    backend_name = env.coredata.get_option(OptionKey('backend'))
+    assert isinstance(backend_name, str)
+    assert backend_name in backend_generator_map
+    return ['-G', backend_generator_map[backend_name]]
+
 def cmake_defines_to_args(raw: T.Any, permissive: bool = False) -> T.List[str]:
     res = []  # type: T.List[str]
     if not isinstance(raw, list):
@@ -103,10 +137,10 @@ def cmake_defines_to_args(raw: T.Any, permissive: bool = False) -> T.List[str]:
                 mlog.warning('  --> Ignoring this option')
                 continue
             if isinstance(val, (str, int, float)):
-                res += ['-D{}={}'.format(key, val)]
+                res += [f'-D{key}={val}']
             elif isinstance(val, bool):
                 val_str = 'ON' if val else 'OFF'
-                res += ['-D{}={}'.format(key, val_str)]
+                res += [f'-D{key}={val_str}']
             else:
                 raise MesonException('Type "{}" of "{}" is not supported as for a CMake define value'.format(type(val).__name__, key))
 
@@ -118,7 +152,7 @@ def check_cmake_args(args: T.List[str]) -> T.List[str]:
     dis = ['-D' + x for x in blacklist_cmake_defs]
     assert dis  # Ensure that dis is not empty.
     for i in args:
-        if any([i.startswith(x) for x in dis]):
+        if any(i.startswith(x) for x in dis):
             mlog.warning('Setting', mlog.bold(i), 'is not supported. See the meson docs for cross compilation support:')
             mlog.warning('  - URL: https://mesonbuild.com/CMake-module.html#cross-compilation')
             mlog.warning('  --> Ignoring this option')
@@ -128,19 +162,19 @@ def check_cmake_args(args: T.List[str]) -> T.List[str]:
 
 class CMakeInclude:
     def __init__(self, path: Path, isSystem: bool = False):
-        self.path     = path
+        self.path = path
         self.isSystem = isSystem
 
     def __repr__(self) -> str:
-        return '<CMakeInclude: {} -- isSystem = {}>'.format(self.path, self.isSystem)
+        return f'<CMakeInclude: {self.path} -- isSystem = {self.isSystem}>'
 
 class CMakeFileGroup:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.defines      = data.get('defines', '')                       # type: str
-        self.flags        = _flags_to_list(data.get('compileFlags', ''))  # type: T.List[str]
+        self.defines = data.get('defines', '')                       # type: str
+        self.flags = _flags_to_list(data.get('compileFlags', ''))  # type: T.List[str]
         self.is_generated = data.get('isGenerated', False)                # type: bool
-        self.language     = data.get('language', 'C')                     # type: str
-        self.sources      = [Path(x) for x in data.get('sources', [])]    # type: T.List[Path]
+        self.language = data.get('language', 'C')                     # type: str
+        self.sources = [Path(x) for x in data.get('sources', [])]    # type: T.List[Path]
 
         # Fix the include directories
         self.includes = []  # type: T.List[CMakeInclude]
@@ -166,21 +200,21 @@ class CMakeFileGroup:
 
 class CMakeTarget:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.artifacts               = [Path(x) for x in data.get('artifacts', [])]         # type: T.List[Path]
-        self.src_dir                 = Path(data.get('sourceDirectory', ''))                # type: Path
-        self.build_dir               = Path(data.get('buildDirectory', ''))                 # type: Path
-        self.name                    = data.get('name', '')                                 # type: str
-        self.full_name               = data.get('fullName', '')                             # type: str
-        self.install                 = data.get('hasInstallRule', False)                    # type: bool
-        self.install_paths           = [Path(x) for x in set(data.get('installPaths', []))] # type: T.List[Path]
-        self.link_lang               = data.get('linkerLanguage', '')                       # type: str
-        self.link_libraries          = _flags_to_list(data.get('linkLibraries', ''))        # type: T.List[str]
-        self.link_flags              = _flags_to_list(data.get('linkFlags', ''))            # type: T.List[str]
-        self.link_lang_flags         = _flags_to_list(data.get('linkLanguageFlags', ''))    # type: T.List[str]
-        # self.link_path             = Path(data.get('linkPath', ''))                       # type: Path
-        self.type                    = data.get('type', 'EXECUTABLE')                       # type: str
-        # self.is_generator_provided = data.get('isGeneratorProvided', False)               # type: bool
-        self.files                   = []                                                   # type: T.List[CMakeFileGroup]
+        self.artifacts = [Path(x) for x in data.get('artifacts', [])]               # type: T.List[Path]
+        self.src_dir = Path(data.get('sourceDirectory', ''))                        # type: Path
+        self.build_dir = Path(data.get('buildDirectory', ''))                       # type: Path
+        self.name = data.get('name', '')                                            # type: str
+        self.full_name = data.get('fullName', '')                                   # type: str
+        self.install = data.get('hasInstallRule', False)                            # type: bool
+        self.install_paths = [Path(x) for x in set(data.get('installPaths', []))]   # type: T.List[Path]
+        self.link_lang = data.get('linkerLanguage', '')                             # type: str
+        self.link_libraries = _flags_to_list(data.get('linkLibraries', ''))         # type: T.List[str]
+        self.link_flags = _flags_to_list(data.get('linkFlags', ''))                 # type: T.List[str]
+        self.link_lang_flags = _flags_to_list(data.get('linkLanguageFlags', ''))    # type: T.List[str]
+        # self.link_path = Path(data.get('linkPath', ''))                             # type: Path
+        self.type = data.get('type', 'EXECUTABLE')                                  # type: str
+        # self.is_generator_provided = data.get('isGeneratorProvided', False)         # type: bool
+        self.files = []                                                             # type: T.List[CMakeFileGroup]
 
         for i in data.get('fileGroups', []):
             self.files += [CMakeFileGroup(i)]
@@ -201,16 +235,16 @@ class CMakeTarget:
         mlog.log('type                  =', mlog.bold(self.type))
         # mlog.log('is_generator_provided =', mlog.bold('true' if self.is_generator_provided else 'false'))
         for idx, i in enumerate(self.files):
-            mlog.log('Files {}:'.format(idx))
+            mlog.log(f'Files {idx}:')
             with mlog.nested():
                 i.log()
 
 class CMakeProject:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.src_dir   = Path(data.get('sourceDirectory', ''))   # type: Path
-        self.build_dir = Path(data.get('buildDirectory', ''))    # type: Path
-        self.name      = data.get('name', '')                    # type: str
-        self.targets   = []                                      # type: T.List[CMakeTarget]
+        self.src_dir = Path(data.get('sourceDirectory', ''))    # type: Path
+        self.build_dir = Path(data.get('buildDirectory', ''))   # type: Path
+        self.name = data.get('name', '')                        # type: str
+        self.targets = []                                       # type: T.List[CMakeTarget]
 
         for i in data.get('targets', []):
             self.targets += [CMakeTarget(i)]
@@ -220,21 +254,21 @@ class CMakeProject:
         mlog.log('build_dir =', mlog.bold(self.build_dir.as_posix()))
         mlog.log('name      =', mlog.bold(self.name))
         for idx, i in enumerate(self.targets):
-            mlog.log('Target {}:'.format(idx))
+            mlog.log(f'Target {idx}:')
             with mlog.nested():
                 i.log()
 
 class CMakeConfiguration:
     def __init__(self, data: T.Dict[str, T.Any]) -> None:
-        self.name     = data.get('name', '')   # type: str
-        self.projects = []                     # type: T.List[CMakeProject]
+        self.name = data.get('name', '')        # type: str
+        self.projects = []                      # type: T.List[CMakeProject]
         for i in data.get('projects', []):
             self.projects += [CMakeProject(i)]
 
     def log(self) -> None:
         mlog.log('name =', mlog.bold(self.name))
         for idx, i in enumerate(self.projects):
-            mlog.log('Project {}:'.format(idx))
+            mlog.log(f'Project {idx}:')
             with mlog.nested():
                 i.log()
 
@@ -265,7 +299,7 @@ class SingleTargetOptions:
             opt = i[:i.find('=')]
             if opt not in self.opts:
                 res += [i]
-        res += ['{}={}'.format(k, v) for k, v in self.opts.items()]
+        res += [f'{k}={v}' for k, v in self.opts.items()]
         return res
 
     def get_compile_args(self, lang: str, initial: T.List[str]) -> T.List[str]:
